@@ -13,6 +13,13 @@ import posixpath
 from flask import current_app, Module, send_from_directory, abort, url_for
 from itertools import chain
 from werkzeug import secure_filename, FileStorage
+try:
+    from flask import Blueprint
+except ImportError:
+    from flask import Module
+    using_blueprints = False
+else:
+    using_blueprints = True
 
 # Extension presets
 
@@ -164,15 +171,18 @@ def configure_uploads(app, upload_sets):
     """
     Call this after the app has been configured. It will go through all the
     upload sets, get their configuration, and store the configuration on the
-    app. It will also register the uploads module if it hasn't been set.
+    app. It will also register the uploads module if it hasn't been set. This
+    can be called multiple times with different upload sets.
+    
+    .. versionchanged:: 0.1.3
+       The uploads module/blueprint will only be registered if it is needed
+       to serve the upload sets.
     
     :param app: The `~flask.Flask` instance to get the configuration from.
     :param upload_sets: The `UploadSet` instances to configure.
     """
     if isinstance(upload_sets, UploadSet):
         upload_sets = (upload_sets,)
-    if '_uploads' not in app.modules:
-        app.register_module(uploads_mod)
     
     if not hasattr(app, 'upload_set_config'):
         app.upload_set_config = {}
@@ -183,6 +193,14 @@ def configure_uploads(app, upload_sets):
     for uset in upload_sets:
         config = config_for_set(uset, app, defaults)
         set_config[uset.name] = config
+    
+    should_serve = any(s.base_url is None for s in set_config.itervalues())
+    if using_blueprints:
+        if '_uploads' not in app.blueprints and should_serve:
+            app.register_blueprint(uploads_mod)
+    else:
+        if '_uploads' not in app.modules and should_serve:
+            app.register_module(uploads_mod)
 
 
 class All(object):
@@ -348,11 +366,17 @@ class UploadSet(object):
         :param storage: The uploaded file to save.
         :param folder: The subfolder within the upload set to save to.
         :param name: The name to save the file as. If it ends with a dot, the
-                     file's extension will be appended to the end.
+                     file's extension will be appended to the end. (If you
+                     are using `name`, you can include the folder in the
+                     `name` instead of explicitly using `folder`, i.e.
+                     ``uset.save(file, name="someguy/photo_123.")``
         """
         if not isinstance(storage, FileStorage):
             raise TypeError("storage must be a werkzeug.FileStorage")
         
+        if folder is None and name is not None and "/" in name:
+            folder, name = name.rsplit("/", 1)
+        
         basename = lowercase_ext(secure_filename(storage.filename))
         if name:
             if name.endswith('.'):
@@ -401,7 +425,10 @@ class UploadSet(object):
                 return newname
 
 
-uploads_mod = Module(__name__, name='_uploads', url_prefix='/_uploads')
+if using_blueprints:
+    uploads_mod = Blueprint('_uploads', __name__, url_prefix='/_uploads')
+else:
+    uploads_mod = Module(__name__, name='_uploads', url_prefix='/_uploads')
 
 @uploads_mod.route('/<setname>/<path:filename>')
 def uploaded_file(setname, filename):
